# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-11-06 07:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0019_auto_20191106_1513'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='item_type',
            field=models.IntegerField(choices=[(1, '\u5b9e\u7269'), (2, '\u865a\u62df')], default=2, verbose_name='\u5546\u54c1\u7c7b\u578b'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='item',
            name='item_type',
            field=models.IntegerField(choices=[(1, '\u5b9e\u7269'), (2, '\u865a\u62df')], default=0, verbose_name='\u5546\u54c1\u7c7b\u578b'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

from django.conf.urls import include, url
from . import views

app_name = 'bee_django_coin'

urlpatterns = [
    url(r'^test$', views.test, name='test'),
    url(r'^$', views.CoinTypeList.as_view(), name='index'),
    # =======发金币类型========
    url(r'^type/list$', views.CoinTypeList.as_view(), name='coin_type_list'),
    # url(r'^message/detail/(?P<pk>[0-9]+)$', views.CoinTypeDetail.as_view(), name='coin_type_detail'),
    url(r'^type/add/$', views.CoinTypeCreate.as_view(), name='coin_type_add'),
    url(r'^type/update/(?P<pk>[0-9]+)/$', views.CoinTypeUpdate.as_view(), name='coin_type_update'),
    url(r'^type/delete/(?P<pk>[0-9]+)/$', views.CoinTypeDelete.as_view(), name='coin_type_delete'),
    # =======发送记录========
    url(r'^user/record/list/(?P<user_id>[0-9]+)/$', views.UserRecordList.as_view(), name='user_record_list'),
    url(r'^user/record/add/(?P<user_id>[0-9]+)/$', views.UserRecordCreate.as_view(), name='coin_record_add'),
    # =======其他类型金币,目前是发班级M币========
    url(r'^other/record/add/(?P<coin_identity>(.)+)/(?P<coin_content_id>[0-9]+)/$', views.OtherRecordCreate.as_view(),
        name='other_record_add'),
    url(r'^other/record/list/(?P<coin_identity>(.)+)/(?P<coin_content_id>[0-9]+)/$', views.OtherRecordList.as_view(),
        name='other_record_list'),
    # =======金币排行榜=======
    url(r'^user/rank/list/$', views.UserRankList.as_view(), name='user_rank_list'),
    url(r'^custom_user/rank/list/$', views.CustomUserRankList.as_view(), name='custom_user_rank_list'),
    # url(r'^user/record/click$', views.UserRecordClick.as_view(), name='user_record_click'),

    # =======商城=======
    # =======商品=======
    url(r'^item/list$', views.ItemList.as_view(), name='item_list'),
    url(r'^item/detail/(?P<pk>[0-9]+)$', views.ItemDetail.as_view(), name='item_detail'),
    url(r'^item/add/$', views.ItemCreate.as_view(), name='item_add'),
    url(r'^item/update/(?P<pk>[0-9]+)/$', views.ItemUpdate.as_view(), name='item_update'),
    url(r'^item/delete/(?P<pk>[0-9]+)/$', views.ItemDelete.as_view(), name='item_delete'),
    # =====用户商品======
    url(r'^item/custom/list/$', views.ItemCustomList.as_view(), name='item_custom_list'),  # 用户的商城商品列表页
    url(r'^item/custom/detail/(?P<pk>[0-9]+)/$', views.ItemCustomDetail.as_view(), name='item_custom_detail'),
    # 用户的商城商品详情页
    # =====订单======
    url(r'^order/list/$', views.OrderList.as_view(), name='order_list'),
    url(r'^order/user/list/(?P<user_id>[0-9]+)/$', views.OrderUserList.as_view(), name='order_user_list'),
    url(r'^order/custom/user/list/(?P<user_id>[0-9]+)/$', views.OrderCustomUserList.as_view(), name='order_custom_user_list'),
    url(r'^order/detail/(?P<pk>[0-9]+)/$', views.OrderDetail.as_view(), name='order_detail'),
    url(r'^order/custom/user/detail/(?P<pk>[0-9]+)/$', views.OrderCustomUserDetail.as_view(), name='order_custom_user_detail'),
    url(r'^order/add/$', views.OrderCreate.as_view(), name='order_add'),
    url(r'^order/admin/add/(?P<user_id>[0-9]+)/$', views.OrderAdminCreate.as_view(), name='order_admin_add'),
    url(r'^order/update/status/$', views.OrderUpdateStatus.as_view(), name='order_update_status'),
    url(r'^order/update/(?P<pk>[0-9]+)/$', views.OrderUpdate.as_view(), name='order_update'),

]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime, pytz
from django.db import models
from django.core.urlresolvers import reverse
from django.conf import settings
from django.contrib.auth.models import User
from django.db.models import Sum, Count
from django.dispatch import receiver
from django.db.models.signals import post_save
from .signals import send_message_signal, add_user_track_signal

LOCAL_TIMEZONE = pytz.timezone('Asia/Shanghai')


# Create your models here.
def get_user_table():
    return settings.AUTH_USER_MODEL


# 获取自定义user的自定义name
def get_user_name(user):
    try:
        return getattr(user, settings.USER_NAME_FIELD)
    except:
        return None


class CoinType(models.Model):
    name = models.CharField(max_length=180, verbose_name='名称类型')
    coin = models.IntegerField(verbose_name="数量")
    info = models.CharField(max_length=180, null=True, blank=True, verbose_name='说明')
    identity = models.CharField(max_length=180, null=True, verbose_name='标识符', unique=True, help_text='此字段唯一')

    class Meta:
        app_label = 'bee_django_coin'
        db_table = 'bee_django_coin_type'
        ordering = ["id"]
        permissions = (
            ('can_manage_coin', '可以进入M币管理页'),
        )

    def __unicode__(self):
        return (self.name)

    def get_absolute_url(self):
        return reverse('bee_django_coin:coin_type_list')


# 缦币
class UserCoinRecord(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='coin_user')
    coin = models.IntegerField(verbose_name='数量', help_text='扣除填入负数')
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="created_by_user", null=True)
    reason = models.CharField(max_length=180, verbose_name='原因', null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    coin_type = models.ForeignKey('bee_django_coin.CoinType', null=True, on_delete=models.SET_NULL, verbose_name='类型')
    coin_content_id = models.IntegerField(null=True)  # 如果coin_type为班级金币，此字段为班级id

    class Meta:
        app_label = 'bee_django_coin'
        db_table = 'bee_django_coin_record'
        ordering = ["-created_at"]

    def __unicode__(self):
        return ("UserCoinRecord->reason:" + self.reason)

    def get_absolute_url(self):
        return reverse('bee_django_coin:user_record_list', kwargs={"user_id": self.user.id})

    def get_created_by_user_name(self):
        if self.created_by:
            return get_user_name(self.created_by)
        else:
            return settings.COIN_DEFAULT_NAME


class UserCoinCount(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    coin_count = models.IntegerField(default=0)

    class Meta:
        app_label = 'bee_django_coin'
        db_table = 'bee_django_coin_rank_list'
        ordering = ["-coin_count"]

    def __unicode__(self):
        return ("UserCoinCount->user:" + self.coin_count.__str__())

    @classmethod
    def update(cls, user):
        user_record = UserCoinRecord.objects.filter(user=user).aggregate(sum_coin=Sum("coin"))
        try:
            user_count = cls.objects.get(user=user)
        except:
            user_count = cls()
            user_count.user = user
        user_count.coin_count = user_record["sum_coin"]
        user_count.save()

        return


# 增加记录后，
# 1.自动更新用户总M币数量
# 2.如果发的是班级m币，自动扣减班级m币总数
@receiver(post_save, sender=UserCoinRecord)
def create_user(sender, **kwargs):
    user_record = kwargs['instance']
    if kwargs['created']:
        send_message_signal.send(sender=UserCoinRecord, user_coin_record=user_record)
        UserCoinCount.update(user_record.user)
        # 更新班级M币总数
        if user_record.coin_type.identity == 'user_class':
            OtherCoinCount.update(coin_type=user_record.coin_type, coin_content_id=user_record.coin_content_id,
                                  count=user_record.coin)
    return


# OTHER_TYPE_CHOICES = ((1, "班级剩余金币"),)


class OtherCoinCount(models.Model):
    # other_type = models.CharField(max_length=180, choices=OTHER_TYPE_CHOICES, null=True)
    coin_type = models.ForeignKey('bee_django_coin.CoinType', null=True, on_delete=models.SET_NULL, verbose_name='类型')
    coin_content_id = models.IntegerField()  # other_type为班级剩余金币，此字段为班级id
    count = models.IntegerField(default=0)  # 金币数
    update_at = models.DateTimeField(auto_now=True)

    class Meta:
        app_label = 'bee_django_coin'
        db_table = 'bee_django_coin_other_count'
        ordering = ["pk"]
        permissions = (
            ('add_teach_class_coin', '可以发所教班级M币'),
        )

    def __unicode__(self):
        return (
            "OtherCoinCount->other_type:" + self.coin_type.name + ",other_type_id:" + self.other_type_id.__str__())

    # 更新M币总数
    @classmethod
    def update(cls, coin_type, coin_content_id, count):
        records = cls.objects.filter(coin_type=coin_type, coin_content_id=coin_content_id)
        if not records.exists():
            return
        record = records.first()
        new_coin = record.count - count
        record.count = new_coin
        record.save()


# ======商城
# 商品
ITEM_TYPE_CHOICES = ((1, "实物"), (2, "虚拟"))
ITEM_STATUS_CHOICES = ((1, "上架"), (2, "下架"))


class Item(models.Model):
    name = models.CharField(max_length=180, verbose_name='商品名称')  # 商品名
    coin = models.IntegerField(verbose_name='购买需要')  # 缦币
    pic = models.ImageField(null=True, blank=True, verbose_name="商品图片", upload_to='bee_django_coin/shop/item')  # 商品图片
    info = models.TextField(null=True, verbose_name='详情', blank=True)  # 详情
    item_type = models.IntegerField(default=0, verbose_name='商品类型', choices=ITEM_TYPE_CHOICES)  # 商品类型 1-实物 2-虚拟
    status = models.IntegerField(default=0, verbose_name="商品状态", choices=ITEM_STATUS_CHOICES)  # 商品状态 1-正常显示，即上架的，2-下架的
    edit_at = models.DateTimeField(auto_now=True)  # 修改时间
    url = models.URLField(null=True,blank=True,verbose_name='商品链接',help_text='填写此项后，点击该商品则自动跳转到链接地址。')

    class Meta:
        app_label = 'bee_django_coin'
        db_table = 'bee_django_coin_item'
        ordering = ["pk"]
        permissions = (
            ('view_item_list', '查看商品列表'),
        )

    def __unicode__(self):
        return ("Item->name:" + self.name)

    def get_absolute_url(self):
        return reverse('bee_django_coin:item_list')

    def get_pic_url(self):
        if not self.pic:
            return '/static/bee_django_coin/img/shop/default_item.jpg'
        return self.pic.url

    def get_seled_count(self):
        total_count = 0
        orders = Order.objects.filter(item_id=self.id, ).aggregate(Sum('item_count'))
        if orders["item_count__sum"]:
            total_count = orders["item_count__sum"]
        return total_count

    def get_status(self):
        if not self.status:
            return ""
        for g in ITEM_STATUS_CHOICES:
            if self.status == g[0]:
                return g[1]
        return ""

    def get_item_type(self):
        if not self.item_type:
            return ""
        for g in ITEM_TYPE_CHOICES:
            if self.item_type == g[0]:
                return g[1]
        return ""


# 订单
ORDER_STATUS_CHOICES = ((1, "待付款"), (2, "已付款"), (3, "已完成"), (4, '已退款'))


class Order(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name='购买人')  # 购买人
    # item = models.ForeignKey('shop.Item', related_name='shop_order_item')  # 购买的商品
    order_id = models.CharField(max_length=180)  # 订单长id 时间+3位随机数+userid
    item_id = models.IntegerField(null=True)  # 商品id
    item_name = models.CharField(max_length=180,verbose_name='商品名称')  # 购买的商品名称
    item_count = models.IntegerField(default=1,verbose_name='商品数量')  # 购买的商品数量
    item_coin = models.IntegerField(default=0)  # 购买的商品总价格
    item_type = models.IntegerField(verbose_name='商品类型', choices=ITEM_TYPE_CHOICES)  # 商品类型 1-实物 2-虚拟
    created_at = models.DateTimeField(auto_now_add=True)  # 订单产生时间
    status = models.IntegerField(default=1)  # 状态
    info = models.TextField(null=True, verbose_name='备注',blank=True)  # 备注信息
    deliver = models.TextField(null=True, verbose_name='发货信息',blank=True,help_text='填写快递单号等信息')  # 备注信息
    evaluate = models.IntegerField(null=True)  # 评价 1好评 -1差评 0中评
    evaluate_info = models.TextField(null=True)  # 评价详情
    evaluate_datetime = models.DateTimeField(null=True)  # 评价时间，已此来判断是否评价

    class Meta:
        app_label = 'bee_django_coin'
        db_table = 'bee_django_coin_order'
        ordering = ["-created_at"]
        permissions=(
            ('view_order_list', '查看订单列表'),
        )

    def __unicode__(self):
        return ("Order->name:" + self.item_name)

    def get_status(self):
        if not self.status:
            return ""
        for g in ORDER_STATUS_CHOICES:
            if self.status == g[0]:
                return g[1]
        return ""

    def get_item_type(self):
        if self.item_type == 1:
            return "实物"
        elif self.item_type == 2:
            return '虚拟'
        else:
            return ''

    # 不检查，直接更改订单状态，及详情
    # 发送站内信
    def change_status_info(self, op_user, status=1):
        info_type = '下单成功'
        # 支付
        if status in [2, "2"]:
            if not self.status == 1:
                return (1, '订单状态错误')

            self.status = 2
            self.save()
            info_type = '支付成功'

        # 消费
        if status in [3, "3"]:
            if not self.status == 2:
                return (1, '订单状态错误')
            info_type = '发货/消费完成'
        self.status = status
        now = datetime.datetime.now(tz=LOCAL_TIMEZONE)
        if not self.info:
            info = ""
        else:
            info = self.info
        info += "<p>由【" + get_user_name(op_user) + "】于" + now.strftime("%Y-%m-%d %H:%M:%S") + info_type + "<p>"
        self.info = info
        self.save()
        # 发站内信，记录足迹
        if self.status in [2, "2"]:
            title = '购买商品:' + self.item_name
            add_user_track_signal.send(sender=Order, title=title, order=self)

        elif self.status in [3, "3"]:
            title = '购买的商品:' + self.item_name + "已发货/消费完成"
            send_message_signal.send(sender=Order, title=title, order=self)
            add_user_track_signal.send(sender=Order, title=title, order=self)
        return (0, info_type)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-11-09 06:04
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bee_django_coin', '0008_auto_20181019_1557'),
    ]

    operations = [
        migrations.CreateModel(
            name='Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, verbose_name='\u5546\u54c1\u540d\u79f0')),
                ('coin', models.IntegerField(verbose_name='\u8d2d\u4e70\u9700\u8981')),
                ('pic', models.TextField(help_text='\u76ee\u524d\u4ec5\u652f\u6301\u7f51\u7edc\u56fe\u7247', null=True, verbose_name='\u5546\u54c1\u56fe\u7247\u7f51\u5740')),
                ('info', models.TextField(null=True, verbose_name='\u8be6\u60c5')),
                ('item_type', models.IntegerField(default=0, verbose_name='\u5546\u54c1\u7c7b\u578b')),
                ('stauts', models.IntegerField(default=0, verbose_name='\u5546\u54c1\u72b6\u6001')),
                ('edit_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ['pk'],
                'db_table': 'bee_django_coin_item',
            },
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order_id', models.CharField(max_length=180)),
                ('item_id', models.IntegerField()),
                ('item_name', models.CharField(max_length=180)),
                ('item_count', models.IntegerField(default=0)),
                ('item_coin', models.IntegerField(default=0)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('status', models.IntegerField(default=1)),
                ('info', models.TextField(null=True, verbose_name='\u5907\u6ce8')),
                ('evaluate', models.IntegerField(null=True)),
                ('evaluate_info', models.TextField(null=True)),
                ('evaluate_datetime', models.DateTimeField(null=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='\u8d2d\u4e70\u4eba')),
            ],
            options={
                'ordering': ['-created_at'],
                'db_table': 'bee_django_coin_order',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-18 07:08
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0006_auto_20181016_1717'),
    ]

    operations = [
        migrations.RenameField(
            model_name='othercoincount',
            old_name='other_type_id',
            new_name='coin_content_id',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-11-22 05:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0014_item_url'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='url',
            field=models.URLField(blank=True, help_text='\u586b\u5199\u6b64\u9879\u540e\uff0c\u70b9\u51fb\u8be5\u5546\u54c1\u5219\u81ea\u52a8\u8df3\u8f6c\u5230\u94fe\u63a5\u5730\u5740\u3002', null=True, verbose_name='\u5546\u54c1\u94fe\u63a5'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-16 08:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0004_auto_20180620_1845'),
    ]

    operations = [
        migrations.CreateModel(
            name='OtherCoinCount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('other_type', models.CharField(choices=[(1, '\u73ed\u7ea7\u5269\u4f59\u91d1\u5e01')], max_length=180, null=True)),
                ('other_type_id', models.IntegerField()),
                ('count', models.IntegerField(default=0)),
                ('update_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ['pk'],
                'db_table': 'bee_django_coin_other_count',
            },
        ),
        migrations.AddField(
            model_name='usercoinrecord',
            name='coin_content_id',
            field=models.IntegerField(null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-11-09 06:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0010_auto_20181109_1415'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item',
            name='stauts',
        ),
        migrations.AddField(
            model_name='item',
            name='status',
            field=models.IntegerField(choices=[(1, '\u4e0a\u67b6'), (2, '\u4e0b\u67b6')], default=0, verbose_name='\u5546\u54c1\u72b6\u6001'),
        ),
        migrations.AlterField(
            model_name='item',
            name='item_type',
            field=models.IntegerField(choices=[(1, '\u5b9e\u7269'), (2, '\u62b5\u7528\u5238')], default=0, verbose_name='\u5546\u54c1\u7c7b\u578b'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-12-27 07:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='cointype',
            name='identity',
            field=models.CharField(max_length=180, null=True, unique=True, verbose_name='\u6807\u8bc6\u7b26'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

from django import forms
from django.conf import settings
from .models import CoinType, UserCoinRecord, Item, Order


# ===== course contract======
class CoinTypeForm(forms.ModelForm):
    class Meta:
        model = CoinType
        fields = ['name', "identity", "coin", "info"]


class UserCoinRecordForm(forms.ModelForm):
    class Meta:
        model = UserCoinRecord
        fields = ["coin", "reason", 'coin_type']


class OtherCoinCreateForm(forms.ModelForm):
    user = forms.ModelChoiceField(label='选择学生', required=True, queryset=None)
    coin = forms.IntegerField(label='数量', required=True, min_value=1)
    reason = forms.CharField(label='原因', required=False)

    class Meta:
        model = UserCoinRecord
        fields = ["user", "coin", "reason"]

    def __init__(self, users, *args, **kwargs):
        super(OtherCoinCreateForm, self).__init__(*args, **kwargs)
        self.fields["user"] = forms.ModelChoiceField(queryset=users, label='选择学生', required=True)


#
class ItemForm(forms.ModelForm):
    ITEM_TYPE_CHOICES = ((1, "实物"),(2, "虚拟"),)
    coin = forms.IntegerField(label='商品价格', help_text='购买所需' + settings.COIN_NAME, required=True, min_value=1)
    item_type = forms.ChoiceField(choices=ITEM_TYPE_CHOICES, label='商品类型')

    # stauts=forms.ChoiceField(label='商品状态',choices=ITEM_STATUS)

    class Meta:
        model = Item
        fields = ["name", "url", "coin", 'pic', 'item_type', 'status', "info"]


class OrderAdminCreateForm(forms.ModelForm):
    class Meta:
        model = Order
        fields = ["item_name", "item_count","item_type", "info"]

class OrderUpdateForm(forms.ModelForm):
    deliver = forms.CharField(label='发货信息',required=True,help_text='填写快递单号等信息')  # 备注信息
    class Meta:
        model = Order
        fields = ["deliver"]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-20 07:42
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bee_django_coin', '0002_cointype_identity'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserCoinCount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('coin_count', models.IntegerField(default=0)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-coin_count'],
                'db_table': 'bee_django_coin_rank_list',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-19 07:57
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0007_auto_20181018_1508'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='othercoincount',
            options={'ordering': ['pk'], 'permissions': (('add_teach_class_coin', '\u53ef\u4ee5\u53d1\u6240\u6559\u73ed\u7ea7M\u5e01'),)},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-20 10:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0003_usercoincount'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cointype',
            options={'ordering': ['id'], 'permissions': (('can_manage_coin', '\u53ef\u4ee5\u8fdb\u5165M\u5e01\u7ba1\u7406\u9875'),)},
        ),
        migrations.AlterField(
            model_name='cointype',
            name='identity',
            field=models.CharField(help_text='\u6b64\u5b57\u6bb5\u552f\u4e00', max_length=180, null=True, unique=True, verbose_name='\u6807\u8bc6\u7b26'),
        ),
        migrations.AlterField(
            model_name='usercoinrecord',
            name='coin',
            field=models.IntegerField(help_text='\u6263\u9664\u586b\u5165\u8d1f\u6570', verbose_name='\u6570\u91cf'),
        ),
        migrations.AlterField(
            model_name='usercoinrecord',
            name='coin_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bee_django_coin.CoinType', verbose_name='\u7c7b\u578b'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-16 09:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0005_auto_20181016_1619'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='othercoincount',
            name='other_type',
        ),
        migrations.AddField(
            model_name='othercoincount',
            name='coin_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bee_django_coin.CoinType', verbose_name='\u7c7b\u578b'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-11-06 07:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0018_auto_20191106_1513'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='info',
            field=models.TextField(blank=True, null=True, verbose_name='\u5907\u6ce8'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

from .utils import get_coin_type
from .models import CoinType, UserCoinRecord, UserCoinCount, OtherCoinCount


# django前台显示本地时间
def filter_local_datetime(_datetime):
    return _datetime


# def get_user(request):
#     return get_login_user(request)


# 操作消息记录，返回结果
def add_coin_record(to_user, reason=None, identity=None, coin=None, count=1, created_by=None):
    try:
        record = UserCoinRecord()
        record.user = to_user
        coin_type = get_coin_type(identity)
        record.coin_type = coin_type
        if coin:
            record.coin = coin * count
        elif coin_type:
            record.coin = coin_type.coin * count
        if reason:
            record.reason = reason
        elif coin_type:
            record.reason = coin_type.name
        record.created_by = created_by
        record.save()
        return record.coin
    except Exception as e:
        print('bee_django_coin->add_coin_record->error:' + e.__str__())
        return None


# 获取学生的M币数量
def get_user_coin(user):
    try:
        coin_count = UserCoinCount.objects.get(user=user)
        return coin_count.coin_count
    except:
        return 0


# 获取班级名称
def get_user_class_name(class_id):
    try:
        from bee_django_user.models import UserClass
        user_class = UserClass.objects.get(id=class_id)
        return user_class.name
    except:
        return ""


def get_class_users(user_class_id):
    try:
        from bee_django_user.models import UserClass
        user_class = UserClass.objects.get(id=user_class_id)
        return user_class.get_students()
    except:
        return None


# 获取所有发M币的班级
# def get_all_classes():
#     try:
#         from bee_django_user.models import UserClass
#         class_list = UserClass.objects.all()
#         return class_list
#     except:
#         return []

# 新增或更新班级M币
def update_user_class_coin(class_id, user_count):
    try:
        coin_type = get_coin_type("user_class")
        pass
    except:
        return
    try:
        record = OtherCoinCount.objects.get(coin_type=coin_type, coin_content_id=class_id)
    except:
        record = OtherCoinCount()
        record.coin_type = coin_type
        record.coin_content_id = class_id
    record.count = user_count * coin_type.coin
    record.save()
    return



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-11-06 07:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0017_auto_20191106_1506'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='item_count',
            field=models.IntegerField(default=1, verbose_name='\u5546\u54c1\u6570\u91cf'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-11-06 07:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0020_auto_20191106_1516'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='deliver',
            field=models.TextField(blank=True, help_text='\u586b\u5199\u5feb\u9012\u5355\u53f7\u7b49\u4fe1\u606f', null=True, verbose_name='\u53d1\u8d27\u4fe1\u606f'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-11-09 06:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0009_item_order'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='info',
            field=models.TextField(blank=True, null=True, verbose_name='\u8be6\u60c5'),
        ),
        migrations.AlterField(
            model_name='item',
            name='pic',
            field=models.TextField(blank=True, help_text='\u76ee\u524d\u4ec5\u652f\u6301\u7f51\u7edc\u56fe\u7247', null=True, verbose_name='\u5546\u54c1\u56fe\u7247\u7f51\u5740'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from __future__ import unicode_literals

import random, datetime
from django.conf import settings
from django.shortcuts import render, get_object_or_404, redirect, reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.decorators import method_decorator
from .decorators import cls_decorator, func_decorator
from django.views.generic import ListView, DetailView, TemplateView
from django.db.models import Q, Sum, Count
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from .models import CoinType, UserCoinRecord, UserCoinCount, OtherCoinCount, Item, Order
from .forms import CoinTypeForm, UserCoinRecordForm, OtherCoinCreateForm, ItemForm,OrderAdminCreateForm,OrderUpdateForm
from django.contrib.auth import get_user_model
from django.contrib import messages
from django.db import transaction
from django.contrib.auth.decorators import login_required, permission_required
from django.http import JsonResponse
from django.utils import timezone

from .signals import send_message_signal
from .utils import get_user_name, get_default_name, get_coin_type
from .exports import get_user_coin, get_user_class_name, get_class_users, add_coin_record

User = get_user_model()


# Create your views here.
# =======course=======
def test(request):
    from exports import update_user_class_coin
    update_user_class_coin(1, 100)
    return


@method_decorator(cls_decorator(cls_name='CoinTypeList'), name='dispatch')
class CoinTypeList(ListView):
    template_name = 'bee_django_coin/coin_type/coin_type_list.html'
    context_object_name = 'coin_type_list'
    paginate_by = 20
    queryset = CoinType.objects.all()


# @method_decorator(cls_decorator(cls_name='MessageDetail'), name='dispatch')
# class CoinTypeDetail(DetailView):
#     model = Message
#     template_name = 'bee_django_message/message/message_detail.html'
#     context_object_name = 'message'


@method_decorator(permission_required('bee_django_coin.add_cointype'), name='dispatch')
class CoinTypeCreate(CreateView):
    model = CoinType
    form_class = CoinTypeForm
    template_name = 'bee_django_coin/coin_type/coin_type_form.html'


@method_decorator(permission_required('bee_django_coin.change_cointype'), name='dispatch')
class CoinTypeUpdate(UpdateView):
    model = CoinType
    form_class = CoinTypeForm
    template_name = 'bee_django_coin/coin_type/coin_type_form.html'


@method_decorator(permission_required('bee_django_coin.delete_cointype'), name='dispatch')
class CoinTypeDelete(DeleteView):
    model = CoinType
    success_url = reverse_lazy('bee_django_coin:coin_type_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


# 发金币记录
@method_decorator(cls_decorator(cls_name='UserRecordList'), name='dispatch')
class UserRecordList(ListView):
    template_name = 'bee_django_coin/record/user_record_list.html'
    context_object_name = 'record_list'
    paginate_by = 20
    queryset = None

    def get_user(self):
        user_id = self.kwargs["user_id"]
        try:
            user_table = get_user_model()
            user = user_table.objects.get(id=user_id)
        except:
            user = None
        return user

    def get_queryset(self):
        self.queryset = UserCoinRecord.objects.filter(user=self.get_user())
        return self.queryset

    def get_context_data(self, **kwargs):
        user = self.get_user()
        context = super(UserRecordList, self).get_context_data(**kwargs)
        context['user'] = user
        context['name'] = get_user_name(user)
        context['default_name'] = get_default_name
        context['sum_coin'] = get_user_coin(user)
        return context


# 发M币，增加一条记录
class UserRecordCreate(CreateView):
    model = UserCoinRecord
    form_class = UserCoinRecordForm
    template_name = 'bee_django_coin/record/form.html'

    def get_user(self):
        return get_object_or_404(get_user_model(), pk=self.kwargs["user_id"])

    def get_context_data(self, **kwargs):
        context = super(UserRecordCreate, self).get_context_data(**kwargs)
        user = self.get_user()
        context['name'] = get_user_name(user)
        return context

    def form_valid(self, form):
        if not self.request.user.has_perm('bee_django_coin.add_usercoinrecord'):
            messages.error(self.request, '没有权限')
            return super(UserRecordCreate, self).form_valid(form)
        user_record = form.save(commit=False)
        user = self.get_user()
        user_record.user = user
        user_record.created_by = self.request.user
        user_record.save()
        return super(UserRecordCreate, self).form_valid(form)


class UserRankList(ListView):
    template_name = 'bee_django_coin/rank/list.html'
    context_object_name = 'user_list'
    paginate_by = 100
    queryset = UserCoinCount.objects.all()

    def get_context_data(self, **kwargs):
        context = super(UserRankList, self).get_context_data(**kwargs)
        return context


class CustomUserRankList(UserRankList):
    template_name = 'bee_django_coin/rank/custom_list.html'


class OtherRecordList(ListView):
    model = UserCoinRecord
    template_name = 'bee_django_coin/other_coin/other_record_list.html'
    context_object_name = 'record_list'
    paginate_by = 20
    queryset = []

    def get_queryset(self):
        coin_identity = self.kwargs["coin_identity"]
        coin_content_id = self.kwargs["coin_content_id"]
        # 班级m币
        if coin_identity == 'user_class':
            self.queryset = UserCoinRecord.objects.filter(coin_type__identity=coin_identity,
                                                          coin_content_id=coin_content_id)
        return self.queryset

    def get_count(self):
        coin_identity = self.kwargs["coin_identity"]
        coin_content_id = self.kwargs["coin_content_id"]
        try:
            record = OtherCoinCount.objects.get(coin_type__identity=coin_identity, coin_content_id=coin_content_id)
            count = record.count
        except:
            count = 0
        return count

    def get_context_data(self, **kwargs):
        context = super(OtherRecordList, self).get_context_data(**kwargs)
        context['count'] = self.get_count()
        coin_identity = self.kwargs["coin_identity"]
        coin_content_id = self.kwargs["coin_content_id"]
        context['coin_content_id'] = coin_content_id
        if coin_identity == 'user_class':
            class_name = get_user_class_name(coin_content_id)
            context['title'] = "【" + class_name + "】"
        return context


class OtherRecordCreate(CreateView):
    model = UserCoinRecord
    form_class = None
    fields = ["coin", "user", "reason"]
    template_name = 'bee_django_coin/other_coin/form.html'

    # def get(self, request, *args, **kwargs):
    #     self.form_class=

    def get_success_url(self):
        return reverse('bee_django_coin:other_record_list', kwargs=self.kwargs)

    def get_count(self):
        coin_identity = self.kwargs["coin_identity"]
        coin_content_id = self.kwargs["coin_content_id"]
        try:
            record = OtherCoinCount.objects.get(coin_type__identity=coin_identity, coin_content_id=coin_content_id)
            count = record.count
        except:
            count = 0
        return count

    def get_context_data(self, **kwargs):
        coin_identity = self.kwargs["coin_identity"]
        coin_content_id = self.kwargs["coin_content_id"]
        context = super(OtherRecordCreate, self).get_context_data(**kwargs)
        context["count"] = self.get_count()
        # 班级剩余m币
        if coin_identity == "user_class":
            user_list = get_class_users(coin_content_id)
            context["form"] = OtherCoinCreateForm(user_list)
        return context

    @transaction.atomic
    def form_valid(self, form):
        if not self.request.user.has_perm("bee_django_coin.add_teach_class_coin"):
            messages.error(self.request, '没有权限')
            return redirect(reverse('bee_django_coin:other_record_add', kwargs=self.kwargs))
        count = self.get_count()
        if count < form.instance.coin:
            messages.error(self.request, '剩余数量不足')
            return redirect(reverse('bee_django_coin:other_record_add', kwargs=self.kwargs))
        coin_identity = self.kwargs["coin_identity"]
        coin_content_id = self.kwargs["coin_content_id"]
        coin_type = get_coin_type(coin_identity)
        form.instance.coin_type = coin_type
        form.instance.coin_content_id = coin_content_id
        form.instance.created_by = self.request.user
        return super(OtherRecordCreate, self).form_valid(form)


# =======商城===========
# ==商品==
class ItemList(ListView):
    template_name = 'bee_django_coin/shop/item/list.html'
    context_object_name = 'item_list'
    paginate_by = 20
    queryset = Item.objects.all()


class ItemDetail(DetailView):
    model = Item
    template_name = 'bee_django_coin/shop/item/detail.html'
    context_object_name = 'item'


@method_decorator(cls_decorator(cls_name='CourseCreate'), name='dispatch')
class ItemCreate(CreateView):
    model = Item
    form_class = ItemForm
    template_name = 'bee_django_coin/shop/item/form.html'


class ItemUpdate(UpdateView):
    model = Item
    form_class = ItemForm
    template_name = 'bee_django_coin/shop/item/form.html'


class ItemDelete(DeleteView):
    model = Item
    success_url = reverse_lazy('bee_django_coin:item_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


# 用户的商城商品列表页
class ItemCustomList(ItemList):
    template_name = 'bee_django_coin/shop/item/custom_list.html'
    queryset = Item.objects.filter(status=1)  # 只显示上架的


class ItemCustomDetail(ItemDetail):
    template_name = 'bee_django_coin/shop/item/custom_detail.html'


# =====订单====

class OrderList(ListView):
    template_name = 'bee_django_coin/shop/order/list.html'
    context_object_name = 'order_list'
    paginate_by = 20

    def get_queryset(self):
        user_id = self.request.GET.get("user_id")
        if not user_id in [0, '0', None]:
            return Order.objects.filter(user_id=user_id)
        return Order.objects.all()

    def get_context_data(self, **kwargs):
        context = super(OrderList, self).get_context_data(**kwargs)
        user_id = self.request.GET.get("user_id")
        if not user_id in [0, '0', None]:
            context["user"] = User.objects.get(id=user_id)
        else:
            context["user"] = None
        return context


class OrderDetail(DetailView):
    model = Order
    template_name = 'bee_django_coin/shop/order/detail.html'
    context_object_name = 'order'
    queryset = Order.objects.all()


class OrderCreate(TemplateView):
    def post(self, request, *args, **kwargs):
        # This method is called when valid form data has been POSTed.
        # It should return an HttpResponse.
        user_id = request.POST.get("user_id")
        item_id = request.POST.get("item_id")
        item_count = request.POST.get("item_count")
        total_coin = request.POST.get("total_coin")

        try:
            item = Item.objects.get(id=item_id)
        except:
            return JsonResponse(data={
                'error': 1,
                'message': '商品错误'
            })
        try:
            user = User.objects.get(id=user_id)
        except:
            return JsonResponse(data={
                'error': 1,
                'message': '用户错误'
            })
        if user.userprofile.is_pause == True:
            return JsonResponse(data={
                'error': 1,
                'message': '到期用户无法购买商品'
            })
        total_coin = int(total_coin)
        user_coin_count = get_user_coin(user)
        if user_coin_count < total_coin:
            return JsonResponse(data={
                'error': 1,
                'message': settings.COIN_NAME.decode("utf-8") + u'数量不足'
            })
        order = Order()
        order.user = user
        now = timezone.now()
        r = random.randint(0, 999)
        order.order_id = now.strftime("%Y%m%d%H%M%S") + str(r).zfill(3) + user.id.__str__()
        order.item_id = item_id
        order.item_name = item.name
        order.item_type = item.item_type
        order.item_count = item_count
        order.item_coin = total_coin
        order.save()
        order.change_status_info(self.request.user, 1)
        reason = '购买' + item_count.__str__() + '份商品:' + item.name
        coin = add_coin_record(to_user=user, reason=reason, identity='coin_item', coin=-total_coin, count=1,
                               created_by=None)
        if coin:
            error, msg = order.change_status_info(self.request.user, 2)
            return JsonResponse(data={
                'error': error,
                'message': msg
            })
        else:
            return JsonResponse(data={
                'error': 1,
                'message': '支付失败'
            })

class OrderAdminCreate(CreateView):
    model = Order
    form_class = OrderAdminCreateForm
    template_name = 'bee_django_coin/shop/order/form.html'

    @transaction.atomic
    def form_valid(self, form):
        if not self.request.user.has_perm("bee_django_coin.add_order"):
            messages.error(self.request, '没有权限')
            return redirect(reverse('bee_django_coin:order_list'))
        user_id = self.kwargs["user_id"]
        user = User.objects.get(id=user_id)
        order = form.instance
        now = timezone.now()
        r = random.randint(0, 999)
        order.order_id = now.strftime("%Y%m%d%H%M%S") + str(r).zfill(3) + user_id.__str__()
        order.user=user
        order.item_coin=0
        order.change_status_info(self.request.user, 1)
        order.change_status_info(user, 2)
        return super(OrderAdminCreate, self).form_valid(form)

    def get_success_url(self):
        return reverse("bee_django_coin:order_list")+"?user_id="+self.kwargs["user_id"]


class OrderUpdateStatus(TemplateView):
    def post(self, request, *args, **kwargs):
        # This method is called when valid form data has been POSTed.
        # It should return an HttpResponse.
        order_id = request.POST.get("order_id")
        status = request.POST.get("status")

        try:
            order = Order.objects.get(id=order_id)
        except:
            return JsonResponse(data={
                'error': 1,
                'message': '订单错误'
            })
        error, msg = order.change_status_info(request.user, status)
        return JsonResponse(data={
            'error': error,
            'message': msg
        })

class OrderUpdate(UpdateView):
    model = Order
    form_class = OrderUpdateForm
    template_name = 'bee_django_coin/shop/order/form.html'
    user= None

    @transaction.atomic
    def form_valid(self, form):
        if not self.request.user.has_perm("bee_django_coin.change_order"):
            messages.error(self.request, '没有权限')
            return redirect(reverse('bee_django_coin:order_list'))
        order = form.instance
        order.change_status_info(self.request.user, 3)
        self.user=order.user
        return super(OrderUpdate, self).form_valid(form)

    def get_success_url(self):
        return reverse("bee_django_coin:order_list") + "?user_id=" + self.user.id.__str__()


class OrderUserList(ListView):
    template_name = 'bee_django_coin/shop/order/user_list.html'
    context_object_name = 'order_list'
    paginate_by = 20
    queryset = None

    def get_user(self):
        return User.objects.get(id=self.kwargs["user_id"])

    def get_context_data(self, **kwargs):
        user = self.get_user()
        context = super(OrderUserList, self).get_context_data(**kwargs)
        context['user'] = user
        return context

    def get_queryset(self):
        user = self.get_user()
        return Order.objects.filter(user=user)


class OrderCustomUserList(OrderUserList):
    template_name = 'bee_django_coin/shop/order/custom_user_list.html'


class OrderCustomUserDetail(OrderDetail):
    template_name = 'bee_django_coin/shop/order/custom_user_detail.html'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-12-19 12:57
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_coin', '0015_auto_20181122_1337'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='item',
            options={'ordering': ['pk'], 'permissions': (('view_item_list', '\u67e5\u770b\u5546\u54c1\u5217\u8868'),)},
        ),
        migrations.AlterModelOptions(
            name='order',
            options={'ordering': ['-created_at'], 'permissions': (('view_order_list', '\u67e5\u770b\u8ba2\u5355\u5217\u8868'),)},
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-

__author__ = 'bee'
import json
from django.conf import settings
from django.apps import apps
from django.contrib.auth.models import User
from django.http import HttpResponse
from .models import CoinType
from django.contrib.auth import get_user_model


# from .models import Message

class JSONResponse(HttpResponse):
    def __init__(self, obj):
        if isinstance(obj, dict):
            _json_str = json.dumps(obj)
        else:
            _json_str = obj
        super(JSONResponse, self).__init__(_json_str, content_type="application/json;charset=utf-8")


#
def get_coin_type(identity):
    coin_type = None
    if identity:
        try:
            coin_type = CoinType.objects.get(identity=identity)
        except:
            coin_type = None
    return coin_type


# def get_user_model():
#     if settings.COIN_USER_TABLE in ["", None]:
#         user_model = User
#     else:
#         app_name = settings.COIN_USER_TABLE.split(".")[0]
#         model_name = settings.COIN_USER_TABLE.split(".")[1]
#         app = apps.get_app_config(app_name)
#         user_model = app.get_model(model_name)
#     return user_model


# 获取登录用户
# def get_login_user(request):
#     if settings.COIN_USER_TABLE in ["", None]:
#         return request.user
#
#     token = request.COOKIES.get('cookie_token', '')
#     # 没有登录
#     if not token:
#         return None
#
#     try:
#         user_table = get_user_model()
#         user = user_table.objects.get(token=token)
#         return user
#     except:
#         return None


# 获取自定义user的自定义name
def get_user_name(user):
    try:
        return getattr(user, settings.USER_NAME_FIELD)
    except:
        return None


def get_default_name():
    return settings.COIN_DEFAULT_NAME

# context
def get_context_coin_name(request):
    context = {
        'context_coin_name': settings.COIN_NAME,
    }

    return context

